#!/usr/bin/env python3

# Copyright      2021  Xiaomi Corporation (authors: Fangjun Kuang)


import kaldifst


def test_std_arc():
    arc = kaldifst.StdArc(ilabel=1, olabel=2, weight=1.5, nextstate=3)
    assert arc.ilabel == 1
    assert arc.olabel == 2
    assert arc.weight == 1.5
    assert arc.nextstate == 3
    assert arc.type == "standard"

    assert kaldifst.StdArc.type == "standard"
    #  arc.type = "hello" # error: it is ready only

    assert isinstance(arc.weight, kaldifst.TropicalWeight)

    arc.weight = 2.0
    assert arc.weight == 2.0

    arc.weight = kaldifst.TropicalWeight(3.0)
    assert arc.weight == 3.0


if __name__ == "__main__":
    test_std_arc()



#!/usr/bin/env python3

# Copyright      2021  Xiaomi Corporation (authors: Fangjun Kuang)


import kaldifst


def test_transducer():
    s = """
    0 1 1 2 0.5
    0 1 2 3 1.5
    1 2 3 5 0.2
    2 0.5
    """
    fst = kaldifst.compile(s, fst_type="vector", arc_type="standard")
    assert isinstance(fst, kaldifst.StdVectorFst)


def test_acceptor():
    s = """
    0 1 a 0.5
    1 0.5
    """
    isym = kaldifst.SymbolTable(name="my_input_symbol")
    isym.add_symbol("a", 1)
    fst = kaldifst.compile(s, acceptor=True, isymbols=isym, keep_isymbols=False)
    print(fst.to_str(is_acceptor=True))
    assert fst.input_symbols is None

    fst.input_symbols = isym

    print(fst.to_str(is_acceptor=True))


def main():
    test_transducer()
    test_acceptor()


if __name__ == "__main__":
    main()



from typing import List, Union

from _kaldifst import (
    Lattice,
    StdVectorFst,
    convert_nbest_to_vector,
    lattice_scale,
    scale_lattice,
    shortest_path,
)


def lattice_to_nbest(
    lat: Union[Lattice, StdVectorFst],
    acoustic_scale: float = 1.0,
    lm_scale: float = 1.0,
    n: int = 1,
) -> List[Union[Lattice, StdVectorFst]]:
    """Work out N-best paths in lattices

    It implements
    https://github.com/kaldi-asr/kaldi/blob/master/src/latbin/lattice-to-nbest.cc

    Args:
      lat:
        The input lattice.
      acoustic_scale:
        Scaling factor for acoustic likelihoods.
      lm_scale:
        Scaling factor for language model scores.
      n:
        Number of distinct paths.
    Returns:
      Return a list of linear FSTs.

    **Example for a StdVectorFst**

    .. literalinclude:: ./code/lattice_to_nbest/ex1.py
       :language: python
       :linenos:
       :caption: lattice_to_nbest for a StdVectorFst

    .. figure:: ./code/lattice_to_nbest/vector-fst.svg
        :alt: vector-fst.svg
        :align: center
        :figwidth: 600px

        Visualization of vector-fst.svg

    .. figure:: ./code/lattice_to_nbest/vector-fst-3best-0.svg
        :alt: vector-fst-3best-0.svg
        :align: center
        :figwidth: 600px

        Visualization of vector-fst-3best-0.svg

    .. figure:: ./code/lattice_to_nbest/vector-fst-3best-1.svg
        :alt: vector-fst-3best-1.svg
        :align: center
        :figwidth: 600px

        Visualization of vector-fst-3best-1.svg

    .. figure:: ./code/lattice_to_nbest/vector-fst-3best-2.svg
        :alt: vector-fst-3best-2.svg
        :align: center
        :figwidth: 600px

        Visualization of vector-fst-3best-2.svg

    **Example for a Lattice**

    .. literalinclude:: ./code/lattice_to_nbest/ex2.py
       :language: python
       :linenos:
       :caption: lattice_to_nbest for a Lattice

    .. figure:: ./code/lattice_to_nbest/lattice.svg
        :alt: lattice.svg
        :align: center
        :figwidth: 600px

        Visualization of lattice.svg

    .. figure:: ./code/lattice_to_nbest/lattice-3best-0.svg
        :alt: lattice-3best-0.svg
        :align: center
        :figwidth: 600px

        Visualization of lattice-3best-0.svg

    .. figure:: ./code/lattice_to_nbest/lattice-3best-1.svg
        :alt: lattice-3best-1.svg
        :align: center
        :figwidth: 600px

        Visualization of lattice-3best-1.svg

    .. figure:: ./code/lattice_to_nbest/lattice-3best-2.svg
        :alt: lattice-3best-2.svg
        :align: center
        :figwidth: 600px

        Visualization of lattice-3best-2.svg
    """
    if lm_scale != 1 or acoustic_scale != 1:
        lat = type(lat)(lat)  # a deep copy
        lat_scale = lattice_scale(lmwt=lm_scale, acwt=acoustic_scale)
        scale_lattice(scale=lat_scale, in_out=lat)

    nbest = shortest_path(lat, n=n)
    return convert_nbest_to_vector(nbest)



from _kaldifst import (
    FloatWeight,
    Lattice,
    LatticeArc,
    LatticeWeight,
    StdArc,
    StdConstFst,
    StdFst,
    StdVectorFst,
    SymbolTable,
    TextNormalizer,
    TropicalWeight,
    add_self_loops,
    arcsort,
    compile,
    compose,
    compose_context,
    connect,
    convert_nbest_to_vector,
    determinize,
    determinize_star,
    divide,
    draw,
    equal_align,
    get_linear_symbol_sequence,
    info,
    invert,
    lattice_scale,
    make_linear_acceptor,
    minimize,
    minimize_encoded,
    plus,
    reverse,
    rmepsilon,
    scale_lattice,
    shortest_path,
    times,
)

from .iterator import ArcIterator, StateIterator
from .lattice_to_nbest import lattice_to_nbest
from .table_types import (
    RandomAccessVectorFstReader,
    SequentialVectorFstReader,
    VectorFstWriter,
)
__version__ = '1.7.11'



#!/usr/bin/env python3

# Copyright      2021  Xiaomi Corporation (authors: Fangjun Kuang)


import kaldifst


# see also https://www.openfst.org/twiki/bin/view/FST/ReverseDoc
def test_reverse():
    isym = kaldifst.SymbolTable()
    isym.add_symbol("eps", 0)
    isym.add_symbol("a", 1)
    isym.add_symbol("b", 2)
    isym.add_symbol("c", 3)
    isym.add_symbol("d", 4)
    isym.add_symbol("f", 5)

    osym = isym
    s = """
        0 0 a a 2
        0 1 a a 1
        1 3
        1 2 b b 3
        1 2 c c 4
        2 2 d d 5
        2 3 d d 6
        3 3 f f 2
        3 2
    """

    fst = kaldifst.compile(s, acceptor=False, isymbols=isym, osymbols=osym)
    s1 = kaldifst.draw(
        fst, acceptor=False, isymbols=isym, osymbols=osym, portrait=True
    )
    print(s1)  # see https://git.io/JSDdH

    fst2 = kaldifst.reverse(fst)

    s2 = kaldifst.draw(
        fst2, acceptor=False, isymbols=osym, osymbols=isym, portrait=True
    )
    print(s2)  # see https://git.io/JSDdh


def main():
    test_reverse()


if __name__ == "__main__":
    main()
